"""
@Author: kang.yang
@Date: 2023/5/12 20:49
"""
import time

from playwright.sync_api import sync_playwright, expect

from kuto.utils.common import screenshot_util
from kuto.utils.log import logger


class PlayWrightDriver:

    def __init__(self, browserName: str, headless: bool = False, state: dict = None):
        self.browserName = browserName
        self.headless = headless

        self.playwright = sync_playwright().start()
        if browserName == 'firefox':
            self.browser = self.playwright.firefox.launch(headless=headless)
        elif browserName == 'webkit':
            self.browser = self.playwright.webkit.launch(headless=headless)
        else:
            self.browser = self.playwright.chromium.launch(headless=headless)
        if state:
            self.context = self.browser.new_context(storage_state=state, no_viewport=True)
        else:
            self.context = self.browser.new_context(no_viewport=True)
        self.page = self.context.new_page()

    def switch_tab(self, locator):
        logger.info("开始切换tab")
        start = time.time()
        with self.page.expect_popup() as popup_info:
            locator.click()
        self.page = popup_info.value
        end = time.time()
        logger.info(f"切换成功，耗时: {end - start}s")

    def visit(self, url):
        logger.info(f"访问页面: {url}")
        start = time.time()
        self.page.goto(url)
        end = time.time()
        logger.info(f"访问成功，耗时: {end - start}s")

    def storage_state(self, path=None):
        logger.info("保存浏览器状态信息")
        start = time.time()
        if not path:
            raise ValueError("路径不能为空")
        self.context.storage_state(path=path)
        end = time.time()
        logger.info(f"保存成功，耗时: {end - start}s")

    @property
    def page_content(self):
        """获取页面内容"""
        start = time.time()
        logger.info("获取页面内容")
        content = self.page.content()
        end = time.time()
        logger.info(f"获取成功: {content}，耗时: {end - start}s")
        return content

    def set_cookies(self, cookies: list):
        logger.info("添加cookie并刷新页面")
        start = time.time()
        self.context.add_cookies(cookies)
        self.page.reload()
        end = time.time()
        logger.info(f"操作成功，耗时: {end - start}s")

    def screenshot(self, file_name=None, position=None):
        return screenshot_util(self.page, file_name=file_name, position=position)

    def close(self):
        logger.info("关闭浏览器")
        start = time.time()
        self.page.close()
        self.context.close()
        self.browser.close()
        self.playwright.stop()
        end = time.time()
        logger.info(f"关闭成功，耗时: {end - start}s")

    def assertTitle(self, title: str, timeout: int = 5):
        logger.info(f"断言页面标题等于: {title}")
        expect(self.page).to_have_title(title, timeout=timeout * 1000)

    def assertUrl(self, url: str, timeout: int = 5):
        logger.info(f"断言页面url等于: {url}")
        expect(self.page).to_have_url(url, timeout=timeout * 1000)

    def assertContent(self, text, count=3):
        logger.info(f"断言页面文本包括: {text}")
        url = self.page.url
        self.page.wait_for_url(url)  # 必须调用这个方法，不然获取content可能会报错
        result = False
        for item in range(count):
            page_content = self.page_content
            if text in page_content:
                result = True
                break
            time.sleep(1)
        assert result, f"页面内容不包含文本: {text}"





"""
@Author: kang.yang
@Date: 2023/8/19 10:42
"""
import cv2

# 读取图像
image = cv2.imread('./image/20230819103646_ocr识别定位-校园场馆.png')

# 获取图像的宽度和高度
height, width, _ = image.shape

# 计算每个切割区域的宽度和高度
sub_width = width // 2
sub_height = height // 2

# 切割图像成上下左右四个等份
top_left = image[0:sub_height, 0:sub_width]
top_right = image[0:sub_height, sub_width:width]
bottom_left = image[sub_height:height, 0:sub_width]
bottom_right = image[sub_height:height, sub_width:width]

cv2.imwrite('top_left.png', top_left)



"""
@Author: kang.yang
@Date: 2023/8/2 20:23
"""
from kuto import Page, WebElem as Elem


class IndexPage(Page):
    """首页"""
    loginBtn = Elem(text='登录/注册')
    patentText = Elem(text='查专利')


class LoginPage(Page):
    """登录页"""
    pwdLoginTab = Elem(text='帐号密码登录')
    userInput = Elem(holder='请输入手机号码')
    pwdInput = Elem(holder='请输入密码')
    licenseBtn = Elem(css="span.el-checkbox__inner", index=1)
    loginBtn = Elem(text='立即登录')
    firstCompanyIcon = Elem(xpath="(//img[@class='right-icon'])[1]")



import time
import typing

from uiautomator2 import UiObject
from uiautomator2.xpath import XPathSelector

from kuto.core.android.driver import AndroidDriver
from kuto.utils.exceptions import NoSuchElementException
from kuto.utils.log import logger


def dict_to_str(source_dict: dict):
    dict_str = '_'.join([f"{key}" for key, value in source_dict.items()])
    return f'{dict_str}_find_fail'


class AdrElem(object):
    """
    安卓元素定义
    """

    def __init__(self,
                 driver: AndroidDriver = None,
                 rid: str = None,
                 cname: str = None,
                 text: str = None,
                 xpath: str = None,
                 index: int = None):
        """
        @param driver: 安卓驱动，必填
        @param rid: resourceId定位
        @param cname: className定位
        @param text: text定位，这种定位方式比较慢
        @param xpath: xpath定位
        @param index: 定位出多个元素时，指定索引，从0开始
        """
        self._driver = driver

        self._kwargs = {}
        if rid is not None:
            self._kwargs["resourceId"] = rid
        if cname is not None:
            self._kwargs["className"] = cname
        if text is not None:
            self._kwargs["text"] = text
        if xpath:
            self._kwargs["xpath"] = xpath
        if index is not None:
            self._kwargs["instance"] = index

    def __get__(self, instance, owner):
        """po模式中element初始化不需要带driver的关键"""
        if instance is None:
            return None

        self._driver = instance.driver
        return self

    def get_element(self, timeout=5, screenshot=True):
        """
        增加截图的方法
        @param timeout: 每次查找时间
        @param screenshot
        @return:
        """
        start = time.time()
        _xpath = self._kwargs.get("xpath")
        if _xpath is not None:
            logger.info(f'查找控件: xpath={_xpath}')
        else:
            logger.info(f'查找控件: {self._kwargs}')
        _element = self._driver.d.xpath(_xpath) if \
            _xpath is not None else self._driver.d(**self._kwargs)

        if _element.wait(timeout=timeout):
            end = time.time()
            logger.info(f"查找成功，耗时: {end - start}s")
            return _element
        else:
            end = time.time()
            logger.info(f"查找失败，耗时: {end - start}s")
            if screenshot:
                self._driver.screenshot(dict_to_str(self._kwargs))
            raise NoSuchElementException(f"控件 {self._kwargs} 查找失败")

    def get_text(self, timeout=3):
        logger.info("获取控件文本属性")
        start = time.time()
        text = self.get_element(timeout=timeout).get_text()
        logger.debug(text)
        end = time.time()
        logger.info(f'耗时: {end - start}s, {text}')
        return text

    def exists(self, timeout=3):
        logger.info("检查控件是否存在")
        start = time.time()
        result = False
        try:
            self.get_element(timeout=timeout, screenshot=False)
        except:
            pass
        else:
            result = True
        finally:
            end = time.time()
            logger.info(f'耗时: {end - start}s, {result}')
            return result

    @staticmethod
    def _adapt_center(e: typing.Union[UiObject, XPathSelector],
                      offset=(0.5, 0.5)):
        """
        修正控件中心坐标
        """
        if isinstance(e, UiObject):
            return e.center(offset=offset)
        else:
            return e.offset(offset[0], offset[1])

    def click(self, timeout=5, screenshot=True):
        logger.info("点击控件")
        start = time.time()
        element = self.get_element(timeout=timeout,
                                   screenshot=screenshot)
        # 这种方式经常点击不成功，感觉是页面刷新有影响
        # element.click()
        x, y = self._adapt_center(element)
        self._driver.d.click(x, y)
        end = time.time()
        logger.info(f'点击成功，耗时: {end - start}s')

    def click_exists(self, timeout=3):
        logger.info("控件存在才点击")
        if self.exists(timeout=timeout):
            self.click()
            logger.info(f"点击成功")
        else:
            logger.info("控件不存在")

    def input(self, text):
        logger.info(f"输入文本: {text}")
        start = time.time()
        self.get_element().set_text(text)
        end = time.time()
        logger.info(f"输入成功，耗时: {end - start}s")

    def input_exists(self, text: str, timeout=3):
        logger.info(f"控件存在才输入: {text}")
        if self.exists(timeout=timeout):
            self.input(text)
            logger.info("输入成功")
        else:
            logger.info("控件不存在")

    def input_pwd(self, text):
        """密码输入框输入有时候用input输入不了"""
        logger.info(f"输入密码: {text}")
        start = time.time()
        self.get_element().click()
        self._driver.d(focused=True).set_text(text)
        end = time.time()
        logger.info(f'输入成功，耗时: {end - start}s')

    def clear(self):
        logger.info("清空输入框")
        start = time.time()
        self.get_element().clear_text()
        end = time.time()
        logger.info(f"清空成功，耗时: {end - start}s")

    def assertExists(self, timeout=3):
        logger.info("断言控件存在")
        status = self.exists(timeout=timeout)
        assert status, "控件不存在"

    def assertText(self, text, timeout=3):
        logger.info(f"断言控件文本属性包括: {text}")
        _text = self.get_text(timeout=timeout)
        assert text in _text, f"文本属性 {_text} 不包含 {text}"






import kuto

from page.adr_page import HomePage, MyPage, SettingPage


# @kuto.feature("安卓测试demo")
# @kuto.story("PO模式")
# class TestPODemo(kuto.Case):
#
#     def start(self):
#         self.hp = HomePage(self.driver)
#         self.mp = MyPage(self.driver)
#         self.sp = SettingPage(self.driver)
#
#     @kuto.title("从首页进入设置页")
#     def test_go_setting(self):
#         self.hp.myTab.click(timeout=10)
#         self.mp.settingBtn.click()
#         self.sp.assertActivity(
#             '.me.MeSettingActivity')
#         self.sp.screenshot("设置页")


@kuto.feature("安卓测试demo")
@kuto.story("普通模式")
class TestNormalDemo(kuto.Case):

    @kuto.title("从首页进入设置页")
    def test_go_setting(self):
        self.elem(xpath='//*[@resource-id="com.qizhidao.clientapp:id/ll'
                        'BottomTabs"]/android.widget.FrameLayout[4]').click(timeout=10)
        self.elem(rid='com.qizhidao.clientapp:id/me_top_bar_setting_iv').click()
        self.assertContent("设置")
        self.screenshot("设置页")


if __name__ == '__main__':
    # 直接执行本文件，需要修改conf.yml中的platform
    kuto.main(
        platform='android',
        did='UJK0220521066836',
        pkg='com.qizhidao.clientapp')





import kuto

from page.ios_page import IndexPage, MyPage


class TestPoDemo(kuto.Case):

    def start(self):
        self.ip = IndexPage(self.driver)
        self.mp = MyPage(self.driver)

    def test_go_setting(self):
        self.ip.myTab.click()
        self.mp.settingBtn.click()
        self.mp.assertContent("设置")


class TestNormalDemo(kuto.Case):

    def test_go_setting(self):
        self.elem(text='我的').click()
        self.elem(text="settings navi").click()
        self.assertContent("设置")
        self.screenshot("设置页")


if __name__ == '__main__':
    # 直接执行本文件，需要修改conf.yml中的platform
    kuto.main(
        platform="ios",
        did='00008101-000E646A3C29003A',
        pkg='com.qizhidao.company',
        auto_start=True)




"""
@Author: kang.yang
@Date: 2023/5/16 14:37
"""
import kuto

from page.web_page import IndexPage, LoginPage


# @kuto.feature("web测试demo")
# @kuto.story("PO模式")
# class TestPoDemo(kuto.Case):
#
#     def start(self):
#         self.ip = IndexPage(self.driver)
#         self.lp = LoginPage(self.driver)
#
#     @kuto.title("登录")
#     def test_login(self):
#         self.open()
#         self.ip.loginBtn.click()
#         self.lp.pwdLoginTab.click()
#         self.lp.userInput.input('13652435335')
#         self.lp.pwdInput.input('wz123456@QZD')
#         self.lp.licenseBtn.click()
#         self.lp.loginBtn.click()
#         self.lp.firstCompanyIcon.click()
#         self.lp.assertContent("查专利")
#         self.screenshot("首页")


@kuto.feature("web测试demo")
@kuto.story("普通模式")
class TestNormalDemo(kuto.Case):

    @kuto.title("登录")
    def test_login(self):
        self.open()
        kuto.logger.info(self.driver.page.url)
        self.sleep(5)
        self.elem(text='登录/注册').click()
        self.elem(text='帐号密码登录').click()
        self.elem(holder='请输入手机号码').input("13652435335")
        self.elem(holder='请输入密码').input("wz123456@QZD")
        self.elem(css="span.el-checkbox__inner", index=1).click()
        self.elem(text='立即登录').click()
        self.elem(xpath="(//img[@class='right-icon'])[1]").click()
        self.assertUrl("https://www-test.qizhidao.com/")
        self.elem(text='查专利').assertVisible()
        self.screenshot("首页")


if __name__ == '__main__':
    # 直接执行本文件，需要修改conf.yml中的platform
    kuto.main(
        platform="web",
        browser="chrome",
        host="https://www-test.qizhidao.com"
    )




import time

from kuto.utils.log import logger
from kuto.core.ocr.ocr_discern import OCRDiscern
from kuto.utils.exceptions import ElementNameEmptyException


class OCRElem(object):
    """ocr识别定位"""

    def __init__(self, driver=None, text: str = None, position: str = None):
        """
        @param driver:
        @param text:
        @param position: top_left、top_right、bottom_left、bottom_right
        """
        self.driver = driver
        self.text = text
        self._position = position

    def __get__(self, instance, owner):
        if instance is None:
            return None

        self.driver = instance.driver
        return self

    def exists(self, retry=3, timeout=1):
        logger.info(f'ocr识别文本: {self.text} 是否存在')
        time.sleep(3)
        for i in range(retry):
            logger.info(f'第{i+1}次查找:')
            image_path = self.driver.screenshot(self.text, position=self._position)
            res = OCRDiscern(image_path).get_coordinate(self.text)
            if isinstance(res, tuple):
                self.driver.screenshot(f'ocr识别定位-{self.text}')
                return True
            time.sleep(timeout)
        else:
            self.driver.screenshot(f'ocr识别定位失败-{self.text}')
            return False

    def click(self, retry=3, timeout=1):
        logger.info(f'ocr点击文本: {self.text}')
        time.sleep(3)
        for i in range(retry):
            logger.info(f'第{i+1}次查找:')
            info = self.driver.screenshot(self.text, position=self._position)
            image_path = info.get("path")
            width = info.get("width")
            height = info.get("height")
            res = OCRDiscern(image_path).get_coordinate(self.text)
            if isinstance(res, tuple):
                x, y = res
                if self._position == 'top_right':
                    x = width/2 + x
                    y = y
                elif self._position == 'top_left':
                    x = x
                    y = y
                elif self._position == 'bottom_left':
                    x = x
                    y = height/2 + y
                elif self._position == 'bottom_right':
                    x = width/2 + x
                    y = height/2 + y
                logger.info(f'识别坐标为: ({x}, {y})')
                self.driver.click(x, y)
                return
            time.sleep(timeout)
        else:
            self.driver.screenshot(f'ocr识别定位失败-{self.text}')
            raise Exception('通过OCR未识别指定文字或置信度过低，无法进行点击操作！')


if __name__ == '__main__':
    from kuto.core.android.driver import AndroidDriver

    driver = AndroidDriver(device_id='UJK0220521066836', pkg_name='com.qizhidao.clientapp')
    # driver.start_app()
    elem = OCRElem(driver, text='体育场馆', position="top_left")
    # all - 850, 938
    # top_right - 307, 938
    # all -
    elem.click()




"""
@Author: kang.yang
@Date: 2023/8/1 18:21
"""
import cv2
import time
import os
import allure

from kuto.utils.log import logger
from kuto.utils.exceptions import ScreenFailException


def cut_by_position(image_path: str, position: str):
    """把图片按左上、左下、右上、右下进行分割"""
    # 读取图像
    image = cv2.imread(image_path)
    # 获取图像的宽度和高度
    height, width, _ = image.shape
    logger.debug(f'{height}, {width}')
    # 计算每个切割区域的宽度和高度
    sub_width = width // 2
    sub_height = height // 2
    # 切割图像成上下左右四个等份
    if position == "top_left":
        image_content = image[0:sub_height, 0:sub_width]
    elif position == "top_right":
        image_content = image[0:sub_height, sub_width:width]
    elif position == "bottom_left":
        image_content = image[sub_height:height, 0:sub_width]
    elif position == "bottom_right":
        image_content = image[sub_height:height, sub_width:width]
    else:
        raise KeyError("position传值错误")

    new_path = f"{image_path.split('.')[0]}_{position}.{image_path.split('.')[1]}"
    logger.debug(new_path)
    cv2.imwrite(new_path, image_content)
    return {
        "path": new_path,
        "height": height,
        "width": width
    }


def screenshot_util(driver, file_name=None, position: str = None):
    """
    截图并保存到预定路径
    @param driver
    @param file_name: foo.png or fool
    @param position: top_left\top_right\bottom_left\bottom_right
    @return:
    """
    logger.info("开始截图")
    start = time.time()
    if not file_name:
        raise ValueError("文件名不能为空")

    # 截图并保存到当前目录的image文件夹中
    relative_path = "image"
    try:
        # 把文件名处理成test.png的样式
        if "." in file_name:
            file_name = file_name.split(r".")[0]
        if os.path.exists(relative_path) is False:
            os.mkdir(relative_path)
        time_str = time.strftime(f"%Y%m%d%H%M%S")
        file_name = f"{time_str}_{file_name}.png"

        file_path = os.path.join(relative_path, file_name)
        # logger.info(f"save to: {os.path.join(relative_path, file_name)}")
        if getattr(driver, "info", None):
            driver.screenshot(file_path)
        else:
            driver.screenshot(path=file_path)
        # 对图片进行分割
        info = None
        if position is not None:
            info = cut_by_position(file_path, position)
            file_path = info.get("path")
        end = time.time()
        logger.info(f"截图完成，保存至: {file_path}，耗时: {end - start}s")
        # 上传allure报告
        allure.attach.file(
            file_path,
            attachment_type=allure.attachment_type.PNG,
            name=f"{file_path}",
        )
        if position is not None:
            return info
        else:
            return file_path
    except Exception as e:
        raise ScreenFailException(f"截图失败: \n{str(e)}")





"""
@Author: kang.yang
@Date: 2023/5/13 10:16
"""
import time

from kuto.utils.exceptions import NoSuchElementException
from kuto.utils.log import logger
from kuto.core.web.driver import PlayWrightDriver
from playwright.sync_api import expect


def dict_to_str(source_dict: dict):
    dict_str = '_'.join([f"{key}" for key, value in source_dict.items()])
    return f'{dict_str}_find_fail'


class WebElem:
    """
    通过selenium定位的web元素
    """

    def __init__(self,
                 driver: PlayWrightDriver = None,
                 xpath: str = None,
                 css: str = None,
                 text: str = None,
                 holder: str = None,
                 index: int = None
                 ):
        """

        @param driver: 浏览器驱动
        @param xpath: xpath
        @param css: css selector
        @param text: 页面内容中的所有文本
        @param holder: 输入框默认文案
        """
        self._driver = driver
        self._xpath = xpath
        self._css = css
        self._text = text
        self._placeholder = holder
        self._index = index

    def __get__(self, instance, owner):
        """pm模式的关键"""
        if instance is None:
            return None

        self._driver = instance.driver
        return self

    def find_element(self):
        element = None
        if self._text:
            logger.info(f"根据文本定位: {self._text}")
            element = self._driver.page.get_by_text(self._text)
        if self._placeholder:
            logger.info(f"根据输入框默认文本定位: {self._placeholder}")
            element = self._driver.page.get_by_placeholder(self._placeholder)
        if self._css:
            logger.info(f"根据css selector定位: {self._css}")
            element = self._driver.page.locator(self._css)
        if self._xpath:
            logger.info(f"根据xpath定位: {self._xpath}")
            element = self._driver.page.locator(self._xpath)
        if self._index:
            element = element.nth(self._index)
        return element

    def fail_handler(self, screenshot=True):
        logger.info("查找失败")
        if screenshot:
            self._driver.screenshot("查找失败")
        raise NoSuchElementException("查找失败")

    def click(self, timeout=10, screenshot=True):
        logger.info("点击控件")
        start = time.time()
        try:
            self.find_element().click(timeout=timeout*1000)
            end = time.time()
            logger.info(f"点击成功，耗时: {end - start}s")
        except:
            end = time.time()
            logger.info(f"点击失败，耗时: {end -start}")
            self.fail_handler(screenshot=screenshot)

    def input(self, text, timeout=10):
        logger.info(f"输入文本: {text}")
        start = time.time()
        try:
            self.find_element().fill(text, timeout=timeout*1000)
            end = time.time()
            logger.info(f"输入成功，耗时: {end - start}s")
        except:
            end = time.time()
            logger.info(f"输入失败，耗时: {end - start}")
            self.fail_handler()

    def check(self, timeout=10):
        logger.info("选择选项")
        start = time.time()
        try:
            self.find_element().check(timeout=timeout*1000)
            end = time.time()
            logger.info(f"选择成功，耗时: {end - start}s")
        except:
            end = time.time()
            logger.info(f"选择失败，耗时: {end - start}")
            self.fail_handler()

    def select(self, value: str, timeout=10):
        logger.info("下拉选择")
        start = time.time()
        try:
            self.find_element().select_option(value, timeout=timeout*1000)
            end = time.time()
            logger.info(f"选择成功，耗时: {end - start}s")
        except:
            end = time.time()
            logger.info(f"选择失败，耗时: {end - start}")
            self.fail_handler()

    def text(self):
        logger.info("获取控件文本属性")
        start = time.time()
        try:
            text = self.find_element().text_content()
            end = time.time()
            logger.info(f"获取成功，耗时: {end - start}s")
            return text
        except:
            end = time.time()
            logger.info(f"获取失败，耗时:{end - start}s")
            self.fail_handler()

    def assertVisible(self):
        logger.info("断言控件可见")
        start = time.time()
        try:
            expect(self.find_element()).to_be_visible()
            end = time.time()
            logger.info(f"断言成功，耗时: {end - start}s")
        except:
            end = time.time()
            logger.info(f"断言失败，耗时: {end - start}s")
            self.fail_handler()

    def assertHidden(self):
        logger.info("断言控件被隐藏")
        start = time.time()
        try:
            expect(self.find_element()).to_be_hidden()
            end = time.time()
            logger.info(f"断言成功，耗时: {end -start}s")
        except:
            end = time.time()
            logger.info(f"断言失败，耗时: {end- start}s")
            self.fail_handler()

    def assertTextContain(self, text: str):
        logger.info(f"断言控件包含文本: {text}")
        start = time.time()
        try:
            expect(self.find_element()).to_contain_text(text)
            end = time.time()
            logger.info(f"断言成功，耗时: {end - start}s")
        except:
            end = time.time()
            logger.info(f"断言失败，耗时: {end - start}s")
            self.fail_handler()

    def assertTextEqual(self, text: str):
        logger.info(f"断言控件文本等于: {text}")
        start = time.time()
        try:
            expect(self.find_element()).to_have_text(text)
            end = time.time()
            logger.info(f"断言成功，耗时: {end - start}")
        except:
            end = time.time()
            logger.info(f"断言失败，耗时: {end - start}")
            self.fail_handler()


class FraElem(WebElem):

    def __init__(self, driver: PlayWrightDriver = None, frame: str = None, xpath: str = None, css: str = None,
                 text: str = None, holder: str = None, index: int = None):
        """

        @param driver: 浏览器驱动
        @type frame: frame定位方式，使用正常的css定位方式即可
        @param xpath: 根据xpath进行定位
        @param css: 根据css selector进行定位
        @param text: 根据标签的文本定位
        @param holder: 根据输入框的placeholder定位
        """
        super().__init__(driver, xpath, css, text, holder, index)
        self._driver = driver
        self._frame_loc = frame
        self._xpath = xpath
        self._css = css
        self._text = text
        self._placeholder = holder
        self._index = index

    def find_element(self):
        element = None
        if self._text:
            logger.info(f"根据文本定位: {self._text}")
            element = self._driver.page.frame_locator(self._frame_loc).get_by_text(self._text)
        if self._placeholder:
            logger.info(f"根据输入框默认文本定位: {self._placeholder}")
            element = self._driver.page.frame_locator(self._frame_loc).get_by_placeholder(self._placeholder)
        if self._css:
            logger.info(f"根据css定位: {self._css}")
            element = self._driver.page.frame_locator(self._frame_loc).locator(self._css)
        if self._xpath:
            logger.info(f"根据xpath定位: {self._xpath}")
            element = self._driver.page.frame_locator(self._frame_loc).locator(self._xpath)
        if self._index:
            element = element.nth(self._index)
        return element



import shutil
import subprocess
import time
import wda

from kuto.utils.exceptions import PkgIsNull, \
    DeviceNotFoundException, WDAStartFail
from kuto.utils.log import logger
from kuto.utils.common import screenshot_util
from kuto.core.ios.common import TideviceUtil


def _start_wda_xctest(udid: str, port, wda_bundle_id=None):
    xctool_path = shutil.which("tidevice")
    args = []
    if udid:
        args.extend(["-u", udid])
    args.append("wdaproxy")
    args.extend(["--port", str(port)])
    if wda_bundle_id is not None:
        args.extend(["-B", wda_bundle_id])
    p = subprocess.Popen([xctool_path] + args)
    time.sleep(3)
    if p.poll() is not None:
        raise WDAStartFail("wda启动失败，可能是手机未连接")


class IosDriver(object):

    def __init__(self, device_id: str = None, pkg_name: str = None, wda_url: str = None):
        if not pkg_name:
            raise PkgIsNull('应用包名不能为空')
        self.pkg_name = pkg_name
        if not device_id and not wda_url:
            raise DeviceNotFoundException('设备id和wda_url不能都为空')
        elif not device_id and wda_url:
            self.wda_url = wda_url
            self.d = wda.Client(self.wda_url)
            if self.d.is_ready():
                logger.info('wda已就绪')
            else:
                logger.info('wda连接失败')
        else:
            self.device_id = device_id
            self.port = self.device_id.split("-")[0][-4:]
            self.wda_url = f"http://localhost:{self.port}"
            self.d = wda.Client(self.wda_url)

            # check if wda is ready
            if self.d.is_ready():
                logger.info('wda已就绪')
            else:
                logger.info('wda未就绪, 现在启动')
                _start_wda_xctest(self.device_id, port=self.port)
                if self.d.is_ready():
                    logger.info('wda启动成功')
                else:
                    raise WDAStartFail('wda启动失败，可能是WebDriverAgent APP端证书失效!')

    def get_device_info(self):
        logger.info("获取设备信息")
        start = time.time()
        info = self.d.device_info()
        end = time.time()
        logger.info(f"{info}，耗时: {end - start}s")
        return info

    def get_current_app(self):
        logger.info("获取当前应用")
        start = time.time()
        info = self.d.app_current()
        end = time.time()
        logger.info(f"{info}，耗时: {end - start}s")

    @property
    def page_content(self):
        logger.info('获取页面xml内容')
        start = time.time()
        page_source = self.d.source(accessible=False)
        end = time.time()
        logger.info(f'耗时: {end - start}s，{page_source}')
        return page_source

    def install_app(self, ipa_url, new=True, pkg_name=None):
        """安装应用
        @param ipa_url: ipa链接
        @param new: 是否先卸载
        @param pkg_name: 应用包名
        @return:
        """
        logger.info(f"安装应用: {ipa_url}")
        start = time.time()
        if new is True:
            pkg_name = pkg_name if pkg_name else self.pkg_name
            self.uninstall_app(pkg_name)

        TideviceUtil.install_app(self.device_id, ipa_url)
        end = time.time()
        logger.info(f"安装成功，耗时: {end - start}s")

    def uninstall_app(self, pkg_name=None):
        start = time.time()
        pkg_name = pkg_name if pkg_name else self.pkg_name
        logger.info(f"卸载应用: {pkg_name}")
        TideviceUtil.uninstall_app(self.device_id, pkg_name)
        end = time.time()
        logger.info(f"卸载成功，耗时: {end - start}s")

    def start_app(self, pkg_name=None, stop=True):
        """启动应用
        @param pkg_name: 应用包名
        @param stop: 是否先停止应用
        """
        start = time.time()
        if not pkg_name:
            pkg_name = self.pkg_name
        logger.info(f"启动应用: {pkg_name}")
        if stop is True:
            self.d.app_terminate(pkg_name)
        self.d.app_start(pkg_name)
        end = time.time()
        logger.info(f"启动成功，耗时: {end - start}s")

    def stop_app(self, pkg_name=None):
        start = time.time()
        if not pkg_name:
            pkg_name = self.pkg_name
        logger.info(f"停止应用: {pkg_name}")
        self.d.app_terminate(pkg_name)
        end = time.time()
        logger.info(f"停止成功，耗时: {end - start}s")

    def back(self):
        """返回上一页"""
        start = time.time()
        logger.info("返回上一页")
        time.sleep(1)
        self.d.swipe(0, 100, 100, 100)
        end = time.time()
        logger.info(f"返回成功，耗时: {end - start}s")

    def screenshot(self, file_name=None, position=None):
        return screenshot_util(self.d, file_name=file_name, position=position)

    def click(self, x, y):
        logger.info(f"点击坐标: ({x}, {y})")
        start = time.time()
        self.d.appium_settings({"snapshotMaxDepth": 0})
        self.d.tap(x, y)
        self.d.appium_settings({"snapshotMaxDepth": 50})
        time.sleep(1)
        end = time.time()
        logger.info(f"点击成功，耗时: {end - start}s")

    def click_alerts(self, alert_list: list):
        """点击弹窗"""
        logger.info(f"点击弹窗: {alert_list}")
        start = time.time()
        try:
            self.d.alert.click(alert_list)
        except:
            pass
        end = time.time()
        logger.info(f"点击成功，耗时: {end - start}s")

    def assertContent(self, text, count=3):
        logger.info(f"断言页面文本包括: {text}")
        result = False
        for item in range(count):
            page_content = self.page_content
            if text in page_content:
                result = True
                break
            time.sleep(1)
        assert result, f"页面内容不包含文本: {text}"







"""
@Author: kang.yang
@Date: 202
3/8/1 16:27
"""
from kuto import Page, IosElem as Elem


class IndexPage(Page):
    """首页"""
    adBtn = Elem(text='close white big')
    myTab = Elem(text='我的')


class MyPage(Page):
    """我的页"""
    settingBtn = Elem(text='settings navi')



"""
@Author: kang.yang
@Date: 2023/8/1 11:53
"""
from kuto import Page, AdrElem as Elem


class DemoPage(Page):
    """定位方式展示"""
    loc_by_rid = Elem(rid="com.qizhidao.clientapp:id/ivScan")
    loc_by_text = Elem(text="我的")
    loc_by_cname = Elem(cname="android.widget.ImageView", index=1)
    loc_by_xpath = Elem(xpath='//*[@resource-id="com.qizhidao.clientapp'
                              ':id/llBottomTabs"]/android.widget.FrameLayout[4]')


class HomePage(Page):
    """APP首页"""
    adBtn = Elem(rid='com.qizhidao.clientapp:id/bottom_btn')
    myTab = Elem(xpath='//*[@resource-id="com.qizhidao.clientapp:id/ll'
                    'BottomTabs"]/android.widget.FrameLayout[4]')


class MyPage(Page):
    """我的页"""
    settingBtn = Elem(rid='com.qizhidao.clientapp:id/me_top_bar_setting_iv')


class SettingPage(Page):
    """设置页"""
    title = Elem(rid='com.qizhidao.clientapp:id/tv_actionbar_title')
    agreementText = Elem(rid='com.qizhidao.clientapp:id/agreement_tv_2')
